@@ -1,14 +1,14 @@
-# $Id: TCP.pm 2220 2007-08-19 00:59:33Z rcaputo $
+# $Id: TCP.pm,v 1.49 2004/11/16 19:21:02 rcaputo Exp $
 
 package POE::Component::Server::TCP;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2220 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my@r=(q$Revision: 1.49 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
 
 use Carp qw(carp croak);
-use Socket qw(INADDR_ANY inet_ntoa inet_aton AF_INET AF_UNIX PF_UNIX);
+use Socket qw(INADDR_ANY inet_ntoa AF_UNIX PF_UNIX);
 use Errno qw(ECONNABORTED ECONNRESET);
 
 # Explicit use to import the parameter constants.
@@ -18,8 +18,6 @@ use POE::Filter::Line;
 use POE::Wheel::ReadWrite;
 use POE::Wheel::SocketFactory;
 
-sub DEBUG () { 0 }
-
 # Create the server.  This is just a handy way to encapsulate
 # POE::Session->create().  Because the states are so small, it uses
 # real inline coderefs.
@@ -42,38 +40,19 @@ sub new {
   # Extract parameters.
   my $alias   = delete $param{Alias};
   my $address = delete $param{Address};
-  my $hname   = delete $param{Hostname};
   my $port    = delete $param{Port};
-  my $domain  = delete($param{Domain}) || AF_INET;
-  my $concurrency = delete $param{Concurrency};
+  my $domain  = delete $param{Domain};
 
   foreach (
     qw(
       Acceptor Error ClientInput ClientConnected ClientDisconnected
       ClientError ClientFlushed
-      ClientLow ClientHigh
     )
   ) {
     croak "$_ must be a coderef"
       if defined($param{$_}) and ref($param{$_}) ne 'CODE';
   }
 
-  my $high_mark_level = delete $param{HighMark};
-  my $low_mark_level  = delete $param{LowMark};
-  my $high_event      = delete $param{ClientHigh};
-  my $low_event       = delete $param{ClientLow};
-
-  my $mark_param_count = (
-    grep { defined $_ }
-    ($high_mark_level, $low_mark_level, $high_event, $low_event)
-  );
-  if ($mark_param_count and $mark_param_count < 4) {
-    croak "If you use the Mark settings, you must define all four";
-  }
-
-  $high_event = sub { } unless defined $high_event;
-  $low_event  = sub { } unless defined $low_event;
-
   my $accept_callback = delete $param{Acceptor};
   my $error_callback  = delete $param{Error};
 
@@ -90,6 +69,7 @@ sub new {
     }
   }
 
+  my (@client_filter_args, @client_infilter_args, @client_outfilter_args);
   my $client_connected    = delete $param{ClientConnected};
   my $client_disconnected = delete $param{ClientDisconnected};
   my $client_error        = delete $param{ClientError};
@@ -104,12 +84,6 @@ sub new {
 
   # Defaults.
 
-  $concurrency = -1 unless defined $concurrency;
-  my $accept_session_id;
-
-  if (!defined $address && defined $hname) {
-    $address = inet_aton($hname);
-  }
   $address = INADDR_ANY unless defined $address;
 
   $error_callback = \&_default_server_error unless defined $error_callback;
@@ -125,6 +99,53 @@ sub new {
   }
 
   if (defined $client_input) {
+    if (defined $client_infilter and defined $client_outfilter) {
+
+      @client_infilter_args  = ();
+      @client_outfilter_args = ();
+
+      if (ref($client_infilter) eq 'ARRAY') {
+         @client_infilter_args = @$client_infilter;
+	 $client_infilter      = shift @client_infilter_args;
+      }
+
+      if (ref($client_outfilter) eq 'ARRAY') {
+         @client_outfilter_args = @$client_outfilter;
+	 $client_outfilter      = shift @client_outfilter_args;
+      }
+
+      my $eval = eval {
+	 (my $inmod  = $client_infilter)  =~ s!::!/!g;
+	 (my $outmod = $client_outfilter) =~ s!::!/!g;
+         require "$inmod.pm";
+         require "$outmod.pm";
+	 1;
+      };
+
+      if (!$eval and $@) {
+        carp "Failed to load [$client_infilter] and [$client_outfilter]\n"
+           . "Reason $@\nUsing defualts ";
+
+        undef($client_infilter);
+        undef($client_outfilter);
+        $client_filter      = "POE::Filter::Line";
+        @client_filter_args = ();
+      }
+
+    } else {
+       undef($client_infilter);  # just to be safe in case one was defined
+       undef($client_outfilter); # and the other wasn't
+
+       unless (defined $client_filter) {
+         $client_filter      = "POE::Filter::Line";
+         @client_filter_args = ();
+       }
+       elsif (ref($client_filter) eq 'ARRAY') {
+         @client_filter_args = @$client_filter;
+         $client_filter      = shift @client_filter_args;
+       }
+    }
+
     $client_error  = \&_default_client_error unless defined $client_error;
     $client_connected    = sub {} unless defined $client_connected;
     $client_disconnected = sub {} unless defined $client_disconnected;
@@ -164,143 +185,122 @@ sub new {
     unless (defined $accept_callback) {
       $accept_callback = sub {
         my ($socket, $remote_addr, $remote_port) = @_[ARG0, ARG1, ARG2];
+        $session_type->create
+          ( @$session_params,
+            inline_states =>
+            { _start => sub {
+                my ( $kernel, $session, $heap ) = @_[KERNEL, SESSION, HEAP];
 
+                $heap->{shutdown} = 0;
+                $heap->{shutdown_on_error} = $shutdown_on_error;
 
-        $session_type->create(
-          @$session_params,
-          inline_states => {
-            _start => sub {
-              my ( $kernel, $session, $heap ) = @_[KERNEL, SESSION, HEAP];
-
-              $heap->{shutdown} = 0;
-              $heap->{shutdown_on_error} = $shutdown_on_error;
-
-              # Unofficial UNIX support, suggested by Damir Dzeko.
-              # Real UNIX socket support should go into a separate
-              # module, but if that module only differs by four
-              # lines of code it would be bad to maintain two
-              # modules for the price of one.  One solution would be
-              # to pull most of this into a base class and derive
-              # TCP and UNIX versions from that.
-              if (
-                $domain == AF_UNIX or $domain == PF_UNIX
-              ) {
-                $heap->{remote_ip} = "LOCAL";
-              }
-              elsif (length($remote_addr) == 4) {
-                $heap->{remote_ip} = inet_ntoa($remote_addr);
-              }
-              else {
-                $heap->{remote_ip} =
-                  Socket6::inet_ntop($domain, $remote_addr);
-              }
-
-              $heap->{remote_port} = $remote_port;
-
-              $heap->{client} = POE::Wheel::ReadWrite->new(
-                Handle       => splice(@_, ARG0, 1),
-                Driver       => POE::Driver::SysRW->new(),
-                _get_filters(
-                  $client_filter,
-                  $client_infilter,
-                  $client_outfilter
-                ),
-                InputEvent   => 'tcp_server_got_input',
-                ErrorEvent   => 'tcp_server_got_error',
-                FlushedEvent => 'tcp_server_got_flush',
-
-                (
-                  $mark_param_count
-                  ? (
-                    HighMark  => $high_mark_level,
-                    HighEvent => 'tcp_server_got_high',
-                    LowMark   => $low_mark_level,
-                    LowEvent  => 'tcp_server_got_low',
-                  )
-                  : ()
-                ),
-              );
-
-              $client_connected->(@_);
-            },
-            tcp_server_got_high => $high_event,
-            tcp_server_got_low => $low_event,
-
-            # To quiet ASSERT_STATES.
-            _child  => sub { },
-
-            tcp_server_got_input => sub {
-              return if $_[HEAP]->{shutdown};
-              $client_input->(@_);
-              undef;
-            },
-            tcp_server_got_error => sub {
-              DEBUG and warn(
-                "$$: $alias child Error ARG0=$_[ARG0] ARG1=$_[ARG1]"
-              );
-              unless ($_[ARG0] eq 'accept' and $_[ARG1] == ECONNABORTED) {
-                $client_error->(@_);
-                if ($_[HEAP]->{shutdown_on_error}) {
-                  $_[HEAP]->{got_an_error} = 1;
-                  $_[KERNEL]->yield("shutdown");
-                }
-              }
-            },
-            tcp_server_got_flush => sub {
-              my $heap = $_[HEAP];
-              DEBUG and warn "$$: $alias child Flush";
-              $client_flushed->(@_);
-              if ($heap->{shutdown}) {
-                DEBUG and warn "$$: $alias child Flush, callback";
-                $client_disconnected->(@_);
-                delete $heap->{client};
-              }
-            },
-            shutdown => sub {
-              DEBUG and warn "$$: $alias child Shutdown";
-              my $heap = $_[HEAP];
-              $heap->{shutdown} = 1;
-              if (defined $heap->{client}) {
+                # Unofficial UNIX support, suggested by Damir Dzeko.
+                # Real UNIX socket support should go into a separate
+                # module, but if that module only differs by four
+                # lines of code it would be bad to maintain two
+                # modules for the price of one.  One solution would be
+                # to pull most of this into a base class and derive
+                # TCP and UNIX versions from that.
                 if (
-                  $heap->{got_an_error} or
-                  not $heap->{client}->get_driver_out_octets()
+                  defined $domain and
+                  ($domain == AF_UNIX or $domain == PF_UNIX)
                 ) {
-                  DEBUG and warn "$$: $alias child Shutdown, callback";
+                  $heap->{remote_ip} = "LOCAL";
+                }
+                elsif (length($remote_addr) == 4) {
+                  $heap->{remote_ip} = inet_ntoa($remote_addr);
+                }
+                else {
+                  $heap->{remote_ip} =
+                    Socket6::inet_ntop($domain, $remote_addr);
+                }
+
+                $heap->{remote_port} = $remote_port;
+
+                # Alter the filters depending whether they are
+                # discrete input/output ones, or whether they are the
+                # same.  It's useful to note that the input/output
+                # filters may be changed at runtime.
+                my @filters;
+                if ($client_infilter) {
+                  @filters = (
+                    InputFilter  => $client_infilter->new(
+                      @client_infilter_args
+                    ),
+                    OutputFilter => $client_outfilter->new(
+                      @client_outfilter_args
+                    ),
+                  );
+                }
+                else {
+                  @filters = (
+                    Filter       => $client_filter->new(@client_filter_args),
+                  );
+                }
+
+                $heap->{client} = POE::Wheel::ReadWrite->new(
+                  Handle       => $socket,
+                  Driver       => POE::Driver::SysRW->new(),
+                  @filters,
+                  InputEvent   => 'tcp_server_got_input',
+                  ErrorEvent   => 'tcp_server_got_error',
+                  FlushedEvent => 'tcp_server_got_flush',
+                );
+
+                $kernel->yield(tcp_server_client_connected => @_[ARG0 .. $#_]);
+              },
+
+              # To quiet ASSERT_STATES.
+              _child  => sub { },
+
+              tcp_server_client_connected => $client_connected,
+
+              tcp_server_got_input => sub {
+                return if $_[HEAP]->{shutdown};
+                $client_input->(@_);
+              },
+              tcp_server_got_error => sub {
+                unless ($_[ARG0] eq 'accept' and $_[ARG1] == ECONNABORTED) {
+                  $client_error->(@_);
+                  if ($_[HEAP]->{shutdown_on_error}) {
+                    $_[HEAP]->{got_an_error} = 1;
+                    $_[KERNEL]->yield("shutdown");
+                  }
+                }
+              },
+              tcp_server_got_flush => sub {
+                my $heap = $_[HEAP];
+                $client_flushed->(@_);
+                if ($heap->{shutdown}) {
                   $client_disconnected->(@_);
                   delete $heap->{client};
                 }
-              }
-            },
-            _stop => sub {
-              ## concurrency on close
-              DEBUG and warn(
-                "$$: $alias _stop accept_session = $accept_session_id"
-              );
-              if( defined $accept_session_id ) {
-                $_[KERNEL]->call( $accept_session_id, 'disconnected' );
-              }
-              else {
-                # This means that the Server::TCP was shutdown before
-                # this connection closed.  So it doesn't really matter that
-                # we can't decrement the connection counter.
-                DEBUG and warn(
-                  "$$: $_[HEAP]->{alias} Disconnected from a connection ",
-                  "without POE::Component::Server::TCP parent"
-                );
-              }
-              return;
+              },
+              shutdown => sub {
+                my $heap = $_[HEAP];
+                $heap->{shutdown} = 1;
+                if (defined $heap->{client}) {
+                  if (
+                    $heap->{got_an_error} or
+                    not $heap->{client}->get_driver_out_octets()
+                  ) {
+                    $client_disconnected->(@_);
+                    delete $heap->{client};
+                  }
+                }
+              },
+              _stop => sub {},
+
+              # User supplied states.
+              %$inline_states
             },
 
-            # User supplied states.
-            %$inline_states
-          },
+            # More user supplied states.
+            package_states => $package_states,
+            object_states  => $object_states,
 
-          # More user supplied states.
-          package_states => $package_states,
-          object_states  => $object_states,
-
-          args => [ $socket, $args ],
-        );
+            args => $args,
+          );
       };
     }
   };
@@ -310,216 +310,73 @@ sub new {
     carp "$mi doesn't recognize \"$_\" as a parameter";
   }
 
-  ## verify concurrency on accept
-  my $orig_accept_callback = $accept_callback;
-  $accept_callback = sub {
-    $_[HEAP]->{connections}++;
-    DEBUG and warn(
-      "$$: $_[HEAP]->{alias} Connection opened ",
-      "($_[HEAP]->{connections} open)"
+  # Create the session, at long last.  This is done inline so that
+  # closures can customize it.
+
+  $session_type->create
+    ( @$session_params,
+      inline_states =>
+      { _start =>
+          sub {
+            if (defined $alias) {
+              $_[HEAP]->{alias} = $alias;
+              $_[KERNEL]->alias_set( $alias );
+            }
+
+            $_[HEAP]->{listener} = POE::Wheel::SocketFactory->new
+              ( BindPort     => $port,
+                BindAddress  => $address,
+                SocketDomain => $domain,
+                Reuse        => 'yes',
+                SuccessEvent => 'tcp_server_got_connection',
+                FailureEvent => 'tcp_server_got_error',
+              );
+	      $server_started->(@_);
+          },
+        # Catch an error.
+        tcp_server_got_error => $error_callback,
+
+        # We accepted a connection.  Do something with it.
+        tcp_server_got_connection => $accept_callback,
+
+        # Shut down.
+        shutdown => sub {
+          delete $_[HEAP]->{listener};
+          $_[KERNEL]->alias_remove( $_[HEAP]->{alias} )
+            if defined $_[HEAP]->{alias};
+        },
+
+        # Dummy states to prevent warnings.
+        _stop   => sub { return 0 },
+        _child  => sub { },
+      },
+
+      args => $args,
     );
-    if( $_[HEAP]->{concurrency} != -1 and $_[HEAP]->{listener} ) {
-      if( $_[HEAP]->{connections} >= $_[HEAP]->{concurrency} ) {
-        DEBUG and warn(
-          "$$: $_[HEAP]->{alias} Concurrent connection limit reached, ",
-          "pausing accept"
-        );
-        $_[HEAP]->{listener}->pause_accept()
-      }
-    }
-    $orig_accept_callback->(@_);
-  };
 
-  # Create the session, at long last.
-  # This is done inline so that closures can customize it.
-  # We save the accept session's ID to avoid self reference.
-
-  $accept_session_id = $session_type->create(
-    @$session_params,
-    inline_states => {
-      _start => sub {
-        if (defined $alias) {
-          $_[HEAP]->{alias} = $alias;
-          $_[KERNEL]->alias_set( $alias );
-        }
-
-        $_[HEAP]->{concurrency} = $concurrency;
-        $_[HEAP]->{connections} = 0;
-
-        $_[HEAP]->{listener} = POE::Wheel::SocketFactory->new(
-          ( ($domain == AF_UNIX or $domain == PF_UNIX)
-            ? ()
-            : ( BindPort => $port )
-          ),
-          BindAddress  => $address,
-          SocketDomain => $domain,
-          Reuse        => 'yes',
-          SuccessEvent => 'tcp_server_got_connection',
-          FailureEvent => 'tcp_server_got_error',
-        );
-        $server_started->(@_);
-      },
-      # Catch an error.
-      tcp_server_got_error => $error_callback,
-
-      # We accepted a connection.  Do something with it.
-      tcp_server_got_connection => $accept_callback,
-
-      # conncurrency on close.
-      disconnected => sub {
-        $_[HEAP]->{connections}--;
-        DEBUG and warn(
-          "$$: $_[HEAP]->{alias} Connection closed ",
-          "($_[HEAP]->{connections} open)"
-        );
-        if ($_[HEAP]->{connections} < 0) {
-          warn(
-            "Excessive 'disconnected' event ",
-            "from $_[CALLER_FILE] at line $_[CALLER_LINE]\n"
-          );
-          $_[HEAP]->{connections} = 0;
-        }
-        if( $_[HEAP]->{concurrency} != -1 and $_[HEAP]->{listener} ) {
-          if( $_[HEAP]->{connections} == ($_[HEAP]->{concurrency}-1) ) {
-            DEBUG and warn(
-              "$$: $_[HEAP]->{alias} Concurrent connection limit ",
-              "reestablished, resuming accept"
-            );
-            $_[HEAP]->{listener}->resume_accept();
-          }
-        }
-      },
-
-      set_concurrency => sub {
-        $_[HEAP]->{concurrency} = $_[ARG0];
-        DEBUG and warn(
-          "$$: $_[HEAP]->{alias} Concurrent connection ",
-          "limit = $_[HEAP]->{concurrency}"
-        );
-        if( $_[HEAP]->{concurrency} != -1 and $_[HEAP]->{listener} ) {
-          if( $_[HEAP]->{connections} >= $_[HEAP]->{concurrency} ) {
-            DEBUG and warn(
-              "$$: $_[HEAP]->{alias} Concurrent connection limit ",
-              "reached, pausing accept"
-            );
-            $_[HEAP]->{listener}->pause_accept()
-          }
-          else {
-            DEBUG and warn(
-              "$$: $_[HEAP]->{alias} Concurrent connection limit ",
-              "reestablished, resuming accept"
-            );
-            $_[HEAP]->{listener}->resume_accept();
-          }
-        }
-      },
-
-      # Shut down.
-      shutdown => sub {
-        delete $_[HEAP]->{listener};
-        $_[KERNEL]->alias_remove( $_[HEAP]->{alias} )
-          if defined $_[HEAP]->{alias};
-      },
-
-      # Dummy states to prevent warnings.
-      _stop   => sub {
-        DEBUG and warn "$$: $_[HEAP]->{alias} _stop";
-        undef($accept_session_id);
-        return 0;
-      },
-      _child  => sub { },
-    },
-
-    args => $args,
-  )->ID;
-
-  # Return the session ID.
-  return $accept_session_id;
-}
-
-sub _get_filters {
-    my ($client_filter, $client_infilter, $client_outfilter) = @_;
-    if (defined $client_infilter or defined $client_outfilter) {
-      return (
-        "InputFilter"  => _load_filter($client_infilter),
-        "OutputFilter" => _load_filter($client_outfilter)
-      );
-      if (defined $client_filter) {
-        carp(
-          "ClientFilter ignored with ClientInputFilter or ClientOutputFilter"
-        );
-      }
-    }
-    elsif (defined $client_filter) {
-     return ( "Filter" => _load_filter($client_filter) );
-    }
-    else {
-      return ( Filter => POE::Filter::Line->new(), );
-    }
-
-}
-
-# Get something: either arrayref, ref, or string
-# Return filter
-sub _load_filter {
-    my $filter = shift;
-    if (ref ($filter) eq 'ARRAY') {
-        my @args = @$filter;
-        $filter = shift @args;
-        if ( _test_filter($filter) ){
-            return $filter->new(@args);
-        } else {
-            return POE::Filter::Line->new(@args);
-        }
-    }
-    elsif (ref $filter) {
-        return $filter->clone();
-    }
-    else {
-        if ( _test_filter($filter) ) {
-            return $filter->new();
-        } else {
-            return POE::Filter::Line->new();
-        }
-    }
-}
-
-# Test if a Filter can be loaded, return sucess or failure
-sub _test_filter {
-    my $filter = shift;
-    my $eval = eval {
-        (my $mod = $filter) =~ s!::!/!g;
-        require "$mod.pm";
-        1;
-    };
-    if (!$eval and $@) {
-        carp(
-          "Failed to load [$filter]\n" .
-          "Reason $@\nUsing defualt POE::Filter::Line "
-        );
-        return 0;
-    }
-    return 1;
+  # Return undef so nobody can use the POE::Session reference.  This
+  # isn't very friendly, but it saves grief later.
+  undef;
 }
 
 # The default server error handler logs to STDERR and shuts down the
 # server.
 
 sub _default_server_error {
-  warn("$$: ".
-    'Server ', $_[SESSION]->ID,
-    " got $_[ARG0] error $_[ARG1] ($_[ARG2])\n"
-  );
+  warn( 'Server ', $_[SESSION]->ID,
+        " got $_[ARG0] error $_[ARG1] ($_[ARG2])\n"
+      );
   delete $_[HEAP]->{listener};
 }
 
-# The default client error handler logs to STDERR
+# The default client error handler logs to STDERR and shuts down the
+# server.
 
 sub _default_client_error {
   my ($syscall, $errno, $error) = @_[ARG0..ARG2];
   unless ($syscall eq "read" and ($errno == 0 or $errno == ECONNRESET)) {
     $error = "(no error)" unless $errno;
-    warn("$$: ".
+    warn(
       'Client session ', $_[SESSION]->ID,
       " got $syscall error $errno ($error)\n"
     );
@@ -538,29 +395,26 @@ POE::Component::Server::TCP - a simplified TCP server
 
   use POE qw(Component::Server::TCP);
 
-  ### First form just accepts connections.
+  # First form just accepts connections.
 
-  my $acceptor_session_id = POE::Component::Server::TCP->new(
+  POE::Component::Server::TCP->new(
     Port     => $bind_port,
     Address  => $bind_address,    # Optional.
-    Hostname => $bind_hostname,   # Optional.
     Domain   => AF_INET,          # Optional.
     Alias    => $session_alias,   # Optional.
     Acceptor => \&accept_handler,
     Error    => \&error_handler,  # Optional.
   );
 
-  ### Second form also handles connections.
+  # Second form accepts and handles connections.
 
-  my $acceptor_session_id = POE::Component::Server::TCP->new(
+  POE::Component::Server::TCP->new(
     Port     => $bind_port,
     Address  => $bind_address,      # Optional.
-    Hostname => $bind_hostname,     # Optional.
     Domain   => AF_INET,            # Optional.
     Alias    => $session_alias,     # Optional.
     Error    => \&error_handler,    # Optional.
     Args     => [ "arg0", "arg1" ], # Optional.
-    Concurrency => -1,              # Optional.
 
     SessionType   => "POE::Session::Abc",           # Optional.
     SessionParams => [ options => { debug => 1 } ], # Optional.
@@ -570,9 +424,9 @@ POE::Component::Server::TCP - a simplified TCP server
     ClientDisconnected => \&handle_client_disconnect, # Optional.
     ClientError        => \&handle_client_error,      # Optional.
     ClientFlushed      => \&handle_client_flush,      # Optional.
-    ClientFilter       => POE::Filter::Xyz->new()",   # Optional.
-    ClientInputFilter  => POE::Filter::Xyz->new(),    # Optional.
-    ClientOutputFilter => POE::Filter::Xyz->new(),    # Optional.
+    ClientFilter       => "POE::Filter::Xyz",         # Optional.
+    ClientInputFilter  => "POE::Filter::Xyz",         # Optional.
+    ClientOutputFilter => "POE::Filter::Xyz",         # Optional.
     ClientShutdownOnError => 0,                       # Optional.
 
     # Optionally define other states for the client session.
@@ -581,7 +435,7 @@ POE::Component::Server::TCP - a simplified TCP server
     ObjectStates  => [ ... ],
   );
 
-  ### Call signatures for handlers.
+  # Call signatures for handlers.
 
   sub accept_handler {
     my ($socket, $remote_address, $remote_port) = @_[ARG0, ARG1, ARG2];
@@ -611,7 +465,7 @@ POE::Component::Server::TCP - a simplified TCP server
     # no special parameters
   }
 
-  ### Reserved HEAP variables:
+  # Reserved HEAP variables:
 
   $heap->{listener}    = SocketFactory (only Acceptor and Error callbacks)
   $heap->{client}      = ReadWrite     (only in ClientXyz callbacks)
@@ -621,18 +475,12 @@ POE::Component::Server::TCP - a simplified TCP server
   $heap->{shutdown}    = shutdown flag (check to see if shutting down)
   $heap->{shutdown_on_error} = Automatically disconnect on error.
 
-  ### Accepted public events.
+  # Accepted public events.
 
-  # Start shutting down this connection.
-  $kernel->yield( "shutdown" );
+  $kernel->yield( "shutdown" )           # initiate shutdown in a connection
+  $kernel->post( server => "shutdown" )  # stop listening for connections
 
-  # Stop listening for connections.
-  $kernel->post( server => "shutdown" );
-
-  # Set the maximum number of simultaneous connections.
-  $kernel->call( server => set_concurrency => $count );
-
-  ### Responding to a client.
+  # Responding to a client.
 
   $heap->{client}->put(@things_to_send);
 
@@ -649,19 +497,10 @@ difficult.  A tutorial at http://poe.perl.org/ describes how.
 
 =head1 CONSTRUCTOR PARAMETERS
 
-=over
-
-=item new
-
-The new() method can accept quite a lot of parameters.  It will return
-the session ID of the accecptor session.  One must use callbacks to
-check for errors rather than the return value of new().
-
+The new() method can accept quite a lot of parameters.
 POE::Component::Server::TCP supplies common defaults for most
 callbacks and handlers.
 
-=back
-
 =over 2
 
 =item Acceptor => CODEREF
@@ -692,7 +531,7 @@ respectively.
 It's passed directly to SocketFactory's BindAddress parameter, so it
 can be in whatever form SocketFactory supports.  At the time of this
 writing, that's a dotted quad, an IPv6 address, a host name, or a
-packed Internet address. See also the Hostname parameter.
+packed Internet address.
 
 =item Alias => SCALAR
 
@@ -708,16 +547,16 @@ Later on, the 'chargen' service can be shut down with:
 
 =item Args => ARRAYREF
 
-Args passes the ARRAYREF to the ClientConnected callback as ARG0.
-It allows you to send extra information to the sessions that handle 
-each client connection.
+Args passes the contents of a ARRAYREF to the ClientConnected callback
+via @_[ARG0..$#_].  It allows you to send extra information to the
+sessions that handle each client connection.
 
 =item ClientConnected => CODEREF
 
 ClientConnected sets the callback used to notify each new client
 session that it is started.  ClientConnected callbacks receive the
 usual POE parameters, plus a copy of whatever was specified in the
-component's C<Args> constructor parameter, as ARG0.
+component's C<Args> constructor parameter.
 
 The ClientConnected callback will not be called within the same
 session context twice.
@@ -743,23 +582,19 @@ context), and so on.
 POE::Wheel::ReadWrite discusses the error parameters in more detail.
 
 A default error handler will be provided if ClientError is omitted.
-The default handler will log most errors to STDERR.
-
-The value of ClientShutdownOnError determines whether the connection
-will be shutdown after errors are received.  It is the client
-shutdown, not the error, that invokes ClientDisconnected callbacks.
+The default handler will log the error to STDERR and close the
+connection.
 
 =item ClientFilter => SCALAR
 
 =item ClientFilter => ARRAYREF
 
-=item ClientFilter => OBJECT
-
 ClientFilter specifies the type of filter that will parse input from
 each client, and optionally any constructor arguments used to create
 each filter instance.
 
-It can take a class name, an arrayref, or a POE::Filter object.
+It takes a POE::Filter class name rather than an object because the
+component must create a new instance for each connection.
 
 If ClientFilter contains a SCALAR, it defines the name of the
 POE::Filter class.  Default constructor parameters will be used to
@@ -774,11 +609,6 @@ line separator to a vertical bar:
 
   ClientFilter => [ "POE::Filter::Line", Literal => "|" ],
 
-If ClientFilter contains an object, it will have a clone method called
-on it each time a client connects.
-
-  ClientFilter => POE::Filter::Block->new()
-
 ClientFilter is optional.  The component will use "POE::Filter::Line"
 if it is omitted.
 
@@ -789,14 +619,10 @@ that filter class.
 
 =item ClientInputFilter => ARRAYREF
 
-=item ClientInputFilter => OBJECT
-
 =item ClientOutputFilter => SCALAR
 
 =item ClientOutputFilter => ARRAYREF
 
-=item ClientOutputFilter => OBJECT
-
 ClientInputFilter and ClientOutputFilter act like ClientFilter, but
 they allow programs to specify different filters for input and output.
 Both must be used together.
@@ -805,7 +631,6 @@ Usage is the same as ClientFilter.
 
   ClientInputFilter  => [ "POE::Filter::Line", Literal => "|" ],
   ClientOutputFilter => "POE::Filter::Stream",
-  ClientInputFilter  => POE::Filter::Block->new(),
 
 =item ClientInput => CODEREF
 
@@ -857,17 +682,6 @@ default handler will log the error to STDERR and shut down the server.
 Active connections will have the opportunity to complete their
 transactions.
 
-=item Hostname => SCALAR
-
-Hostname is the optional non-packed name of the interface the TCP
-server will bind to. This will always be converted via inet_aton
-and so can either be a dotted quad or a name. If you know that you
-are passing in text, then this parameter should be used in preference
-to Address, to prevent confusion in the case that the hostname
-happens to be 4 bytes in length. In the case that both are
-provided, then the Address parameter overrides
-the Hostname parameter.
-
 =item InlineStates => HASHREF
 
 InlineStates holds a hashref of callbacks to handle custom events.
@@ -926,41 +740,6 @@ Its parameters are the usual for a session's _start handler.
 
 Started is optional.
 
-=item Concurrency => SCALAR
-
-Controls the number of connections that may be open at the same time.
-Defaults to -1, which means unlimited number of simutaneous connections.
-0 means no connections.  This value may be set via the
-C<set_concurrency> event, see L<EVENTS>.
-
-Note that if you define the C<Acceptor> callback, you will have to inform
-the TCP server session that a connection was closed. This is done by sending
-a C<disconnected> event to your session's parent. This is only necessary if
-you define an C<Acceptor> callback.  For C<ClientInput>, it's all handled
-for you.
-
-Example:
-
-  Acceptor => sub {
-    # ....
-    POE::Session->create(
-      # ....
-      inline_states => {
-        _start => sub {
-          # ....
-          # remember who our parent is
-          $_[HEAP]->{server_tcp} = $_[SENDER]->ID;
-          # ....
-        },
-        got_client_disconnect => sub {
-          # ....
-          $_[KERNEL]->post( $_[HEAP]->{server_tcp} => 'disconnected' );
-          # ....
-        }
-      }
-    );
-  }
-
 =back
 
 =head1 EVENTS
@@ -978,17 +757,6 @@ if one is set.
 
 Active connections are not shut down until they disconnect.
 
-=item disconnected
-
-Inform the TCP server that a connection was closed.  This is only necessary
-when using C<Concurrency> is set and you are using an L<Acceptor> callback.
-
-=item set_concurrency
-
-Set the number of simultaneous connections.  See L<Concurrency> above.
-
-  $kernel->call( "tcp_server_alias", "set_concurrency", $max_count );
-
 =back
 
 =head1 SEE ALSO
@@ -1004,19 +772,15 @@ This component currently does not accept many of the options that
 POE::Wheel::SocketFactory does.
 
 This component will not bind to several addresses at once.  This may
-be a limitation in SocketFactory, but it's not by design.
+be a limitation in SocketFactory.
 
 This component needs more complex error handling which appends for
 construction errors and replaces for runtime errors, instead of
 replacing for all.
 
-Some use cases require different session classes for the listener and
-the connection handlers.  This isn't currently supported.  Please send
-patches. :)
-
 =head1 AUTHORS & COPYRIGHTS
 
-POE::Component::Server::TCP is Copyright 2000-2006 by Rocco Caputo.
+POE::Component::Server::TCP is Copyright 2000-2001 by Rocco Caputo.
 All rights are reserved.  POE::Component::Server::TCP is free
 software, and it may be redistributed and/or modified under the same
 terms as Perl itself.
@@ -1028,6 +792,3 @@ POE::Component::Server::TCP is based on code, used with permission,
 from Jos Boumans E<lt>kane@cpan.orgE<gt>.
 
 =cut
-
-# rocco // vim: ts=2 sw=2 expandtab
-# TODO - Redocument.
